{*********************************************************}
{*                  FileIO.Pas 1.01                      *}
{*     Copyright (c) 1997 GB Consultancy Heerlen BV      *}
{*                 All rights reserved.                  *}
{*********************************************************}

{FileIo unit was written by:
         *********************************
         *     Paul Backus               *
         *     Leuvenlaan 66             *
         *     6229 GZ  Maastricht       *
         *     Netherlands               *
         *     Tel +31-(0)43-361 10 60   *
         *     email:                    *
         *     paul.backus@writeme.com   *
         *********************************

 This unit can be freely distributed as is.
 The autor is not responsible for any damage or whatever caused by usage
 of this unit,sourcecode etc.

 Usage of this unit obligates:
 1. user of notifying the autor by email. Please state where you found it
 and what you think of it.
 2. user of notifying autor of any bugs found or any new enhancement you made.
}

{
 Version 1.01
  - Added Error trapping for Illegal filepath and serveraddres\path
}


unit FileIo;
Interface

Uses
  Windows, Forms,Controls, SysUtils, Dialogs;


Const
  FileExistsResult : Integer =0;

Type
  ShareModes    = (CompMode,DenyAll,DenyWrite,DenyRead,DenyNone);
  AccessModes   = (Read,Write,ReadWrite);

Var
  IoCode : Integer;

Function  MakeDir(Naam: String;
                  Msg : Boolean):Boolean;

Function  SwitchDir(Dir   : String;
                    Msg   ,
                    Reset : Boolean) : Boolean;

Procedure OkBox(S       : String;
                MsgType : TMsgDlgType);

Function  YesNoBox(S       : String;
                   MsgType : TMsgDlgType) : Integer;

Function  FileExists(     FilePathPlusName : String;
                      Var Filvar           : File;
                          FLen             : Word;
                          AccMode          : AccessModes;
                          Shmode           : Sharemodes;
                          FCreate          ,
                          FClose           ,
                          FileShouldExist  : Boolean) : Boolean;

{Flen = length of block when using BlockWrite, BlockRead
 AccMode : AccessModes (See File open mode constants in Sysutiles Delphi help)
 ShMode  : ShareModes (See File open mode constants in Sysutiles Delphi help)
 Fcreate : Should file be created if not exist
 FClose  : Should file be closed if exist
 FileShouldExist Toggles result accordingly}

Function EraseFile(AFile      : String;
                   CheckExist : Boolean;
                   AMsg       : Boolean) : Boolean;



implementation


Procedure OkBox(S       : String;
                MsgType : TMsgDlgType);

// MsgType mtWarning,mtError,mtInformation,mtConfirmation,mtCustom

Begin
  If S<>'' Then
    MessageDlg(S,MsgType, [MbYes],0);
End;


Function  YesNoBox(S       : String;
                   MsgType : TMsgDlgType) : Integer;

// MsgType mtWarning,mtError,mtInformation,mtConfirmation,mtCustom

Begin
  If S<>'' Then
    Result := MessageDlg(S,MsgType, [MbYes,MbNo],0);
End;


function SwitchDir(Dir   : string;
                   Msg   ,
                   Reset : boolean) : boolean;

{ If Msg then if an error occurs we display a message
  If Reset we go back to the dir we where when we started this function}

var DefaultPath:string;

begin
  GetDir(0,DefaultPath);
  if length(dir)>3 then
    if dir[Length(Dir)] ='\' then
        delete(Dir,Length(dir),1);
  SwitchDir:=false;
  {$I-}
  ChDir(Dir);
  {$I+}
  if IoResult=0 then
     SwitchDir:=true
  else
  if Msg then OkBox('Directory : '+dir+' doesnot exist.',mtError);
  if Reset then ChDir(DefaultPath);
end;


Function MakeDir(Naam : String;
                 Msg  : Boolean) : Boolean;

{ If Msg then if an error occurs we display a message}

Begin
  MakeDir:=True;
  {$I-}
  MkDir(naam);
  {$I+}
  if IoResult<>0 then
      Begin
        If Msg then Okbox('Make Dir.: '+naam,mtError);
        MakeDir:=false;
      End;
End;


function GetFileExistsExtended(FilePathPlusName : String;
                               Var Filvar       : File;
                               FLen             : Word;
                               AccMode          : AccessModes;
                               Shmode           : Sharemodes;
                               FCreate          ,
                               FClose           ,
                               FileShouldExist  : Boolean) : Boolean;


{Results of FileExistsExtended in FileExistResult
  0 = File Exists and is Created if FCreate is True.
  1 = File Doesnot Exist and is not Created.
  2 = File does not Exist but couldnot be created
  3 = File is Created and Closed.
  4 = We have a lock error
  5 = We have a unidentified ioError
  6 = We Have a none existing ServerFilePath;
  7 = We Have a illegal Path
}

Var
  OpenMode  : Integer;
  IoCode    : Integer;
  FileName  : String;
  SaveCursor: Integer;

begin
  //Init Part of m_FileExists;
  OpenMode:=0;
  FileExistsResult:=0;
  GetFileExistsExtended:=False;
  SaveCursor:=Screen.Cursor;
  Screen.Cursor:=crHourGlass;


  Case AccMode of
    Read      : OpenMode := FmOpenRead;
    Write     : OpenMode := FmOpenWrite;
    ReadWrite : OpenMode := FmOpenReadWrite;
    Else  OpenMode:= FmOpenReadWrite;
  End;

  Case ShMode of
    DenyAll    : OpenMode:=OpenMode or fmShareExclusive;
    DenyNone   : OpenMode:=OpenMode or fmShareDenyNone;
    DenyWrite  : OpenMode:=OpenMode or fmShareDenyWrite;
    DenyRead   : OpenMode:=OpenMode or fmShareDenyRead;
    CompMode   : OpenMode:=OpenMode or fmShareComPat;
    Else  OpenMode:=OpenMode or fmShareDenyNone;
  End;

  //Checkpart of m_FileExists;

  Try
    {$I-}
    Repeat
      FileName:=FilePathPlusName;
      AssignFile(Filvar, FileName);
      FileMode := OpenMode;  //Set file access to AccMode+ShMode
      Reset(Filvar,FLen);
      IoCode:=IoResult;

      Case IoCode Of
            0 : // File exists
              Begin
                If FClose Then CloseFile(Filvar); //Leave Open is required
                FileExistsResult:=0;
                Result := True;
              End; // End Of If IoCode = 0

            2  : //File doesnot exist
              Begin
                If FCreate Then
                  Begin
                    Rewrite(Filvar,1);
                    If Ioresult<>0 Then
                      Begin
                        FileExistsresult:=2;
                        Iocode:=0; //Jump out Repeat..Until;
                      End // End of If Ioresult<>0

                    Else //If Ioresult=0 File Created OK

                      Begin
                        CloseFile(Filvar);
                        If FClose Then  // So we can Leave otherwise file should be open at correct file-length
                          Begin
                            FileExistsResult:=3;//File Now Exists and is closed
                            Result:=True;
                            Iocode:=0; //Jump out Repeat..Until;
                          End;
                      End; // End of else

                  End// End of if Ioresult=0

                Else // If Not FCreate Then It's ok file doesnot exist.

                  Begin
                    FileExistsResult:=1;
                    Result:=False;  //It's suppossed to be not there.
                    Iocode:=0; //Jump out Repeat..Until;
                  End;

              End; // End of case 2

            3 : //Path does not exist
              Begin
                Iocode:=0; //Jump out Repeat..Until;
                FileExistsResult:=7;
              End; //End of case 3

            32 :  //We have a locking error;
              Begin
                Iocode:=0; //Jump out Repeat..Until;
                FileExistsResult:=4;
              End; // End of Case 32

            53 : //ServerAddress does not exist
              Begin
                Iocode:=0; //Jump out Repeat..Until;
                FileExistsResult:=6;
              End;

            Else  //We have a unknow file io-error.

              Begin
                FileExistsResult:=5;
                FileIo.IoCode:=IoCode; //set global IoCodeResult
                Iocode:=0;
              End;

      End; // End of Case IoCode Of;
    Until (IoCode=0); //We only loop is file was created and should be left open
  {$I+}
  Finally
    Screen.Cursor:=SaveCursor;
  end;  //End of Try..Finally
End;


function FileExists(    FilePathPlusName : String;
                    Var Filvar           : File;
                        FLen             : Word;
                        AccMode          : AccessModes;
                        Shmode           : Sharemodes;
                        FCreate          ,
                        FClose           ,
                        FileShouldExist  : Boolean) : Boolean;

Var
  Status : Boolean;
  Abort  : Boolean;

Begin
  Abort:=false;
  Repeat
    Status:=GetFileExistsExtended(FilePathPlusName,
                                  Filvar,
                                  FLen,
                                  AccMode,
                                  ShMode,
                                  FCreate,
                                  FClose,
                                  FileShouldExist);
    Case FileExistsResult of
      1 ,
      3 : Abort:=True; // 1 = file doesnot exists 3 = file was created and closed

      2 :  //File Couldnot be created. Creation Error
        Begin
            If YesNoBox('Error in creating :'+
                         ExtractFileName(FilePathPlusName)+
                         #13+#10+'Try Again ?',
                         mtError)= mrNo Then

              Begin
                If FileShouldExist Then
                   Status:=False
                Else
                  Status:=True;
                Abort:=True;
              End;
        End;

      4 : // We Have a LockError !!
        Begin
          If YesNoBox('Error opening :'+
                       ExtractFileName(FilePathPlusName)+
                       #13+#10+
                      'File is in use.'+
                      ' Try Again ?',
                       mtError)= mrNo Then
             Begin
               If FileShouldExist Then
                 Status:=False
               Else
                 Status:=True;
               Abort:=True; //Jump out Repeat..Until;
             End;
        End;
      5 :  //We have a unknow IoError
        Begin
          OkBox(Format('FileExists.IoCode Unknow : %d',[IoCode])+
                       #13+#10+'Contact Autor.',mtError);
          If FileShouldExist Then
            Status:=False
          Else
            Status:=True;
          Abort:=True;
        End;
      6 : //We have a illegal Server Address \\Server\Source\Path
        Begin
          OkBox('Illegal ServerAddress : '+
                 FilePathPlusName,
                 mtError);
          If FileShouldExist Then
            Status:=False
          Else
            Status:=True;
          Abort:=True;
        End;
      7 : //We have A illegal filepath
        Begin
          OkBox('Illegal Filepath : '+
                 ExtractFilePath(FilePathPlusName),
                 mtError);
          If FileShouldExist Then
            Status:=False
          Else
            Status:=True;
          Abort:=True;
        End;
    End;
  Until (Abort) or ((Status) and (FileExistsResult=0));;
  Result:=Status
End;


Function EraseFile(AFile      : String;
                   CheckExist : Boolean;
                   AMsg       : Boolean) : Boolean;

Var
  Filvar : File;
  KillOk : Boolean;

Procedure KillTheFile;

Begin
  Try
    AssignFile(Filvar,AFile);
    {$i-}
    Erase(Filvar);
    KillOk:=Ioresult=0;
    {$I+}
  Finally
  End;
End;


Begin
  KillOk:=False;
  If CheckExist Then
    Begin
      If FileExists(AFile,
                    Filvar,
                    1,
                    ReadWrite,
                    DenyAll,
                    False,
                    True,
                    True) Then
        Begin
          KillTheFile;
        End;
    End
  Else
    KillTheFile;
  If Not(KillOk) Then
    if AMsg Then
      OkBox('Error Deleting :'+ExtractFileName(AFile),mtError);
  Result:=KillOk;
End;

end.


